// TODO: wrap in closure

// global variable _modules with all loaded modules
var _modules = [];

var _module = null;
var _scripts_configs = null;

var automationRoot = "automation/";
var configFile = automationRoot + "config.js";
var moduleFolder = automationRoot + "modules/";

function startAutomation() {
  // load global varible _scripts_configs
  debugPrint("Starting JS Automation engine");
  _scripts_configs = null;
  executeFile(configFile);
  loadAndInitModules();
  _scripts_configs = null;
}

function stopAutomation() {
  stopModules();
}

function loadAndInitModules() {
  for (var i in _scripts_configs) {
    script = _scripts_configs[i];
    debugPrint("Loading module: " + script.module);
    var mod = loadModule(moduleFolder + script.module);
    if (mod) {
      initModule(mod, script.config);
    }
  }
}

function stopModules() {
  for (var i in _modules) {
    _modules[i].stop();
  }
}

function initModule(mod, config) {
  try {
    mod.init(fillDefaults(config, mod.config));
  } catch(e) {
    debugPrint("Can not init module " + mod.name + ": " + e);
  }
}

function loadModule(moduleName) {
  // global variable _module will be a temp variable hosting last loaded module
  _module = getModule(moduleName);
  
  if (!_module) {
    try {
      executeFile(moduleName);
      if (!_module.name || !_module.config || typeof(_module.init) != "function")
        throw "Module syntax is not compleate";
      _modules.push(_module);
    } catch(e) {
      debugPrint("Error loading module: " + e);
      return null;
    }
  }
  
  var mod = _module;
  _module = null; // clean global variable module

  return mod;
}

function getModule(moduleName) {
  for (var i in _modules) {
    if (moduleName == _modules[i].name)
      return _modules[i];
  }
  return null;
}

function fillDefaults(config, config_defaults) {
  for (var param in config_defaults) {
    if (typeof(config[param]) == "undefined") {
      config[param] = config_defaults[param];
    }
  }
  return config;
}


// =========================================

// Private functions
arrayContains = function(array, element) { 
  return array.indexOf(element) !== -1;
}
  
// Public functions

// Device object

function Device(id, tags) {
  this.id = id;
  this.tags = tags.split(' ');
  this.z = zway.devices[id];
}

// Devices object

Devices = function(devicesArray) {
  this.list = devicesArray;
}

Devices.prototype.byId = function(id) {
  return new Devices(this.list.filter(function(element, index, array) { return element.id == id; }));
}

Devices.prototype.self = function() {
  return new Devices(this.list.filter(function(element, index, array) { return element.id == zway.controller.data.nodeId; }));
}

Devices.prototype.filter = function(tags) {
  var tags_array = tags.split(' ');
  //var tags_positive = tags_array.filter(function(element, index, array) { element[0] != '-'}); // for future negation with -. a , is also worth adding for union
  //var tags_negative = tags_array.filter(function(element, index, array) { element[0] == '-'});
  return new Devices(this.list.filter(function(element, index, array) { for (i in tags) { if (!arrayContains(element.tags, tags_array[i])) { return false; }; return true; } }));
}

Devices.prototype.apply = function(func) {
  for (var i = 0; i < this.list.length; i++) {
    func.call(this.list[i]);
  }
}

// ======================================================

executeFile(automationRoot + "/" + "tags.js");
startAutomation();
